/*
 *  main.c
 *  looper
 *
 *  Created by minifloat on 28 Feb 2011
 */


#include <avr/io.h>
#include <inttypes.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>

#include "bassdrum.h"
#include "snare.h"
#include "hihat.h"


inline void set_output(uint8_t val)
{
   OCR1B = (255 - val);
   OCR1A = val;
}

volatile PGM_P playposition = bassdrum;
volatile uint16_t counter = 0;
volatile uint8_t drumcounter = 0;

static uint8_t drumbeat[33] = "B H S H B HSS H B H S H B H SH H";
//                             0123456789ABCDEF0123456789ABCDEF

ISR(TIMER1_OVF_vect)
{
   uint8_t helper;
   
   if (!(counter & 0x03))
   {
      helper = pgm_read_byte(playposition);
      if (helper != 0x00)
      {
         set_output(helper);
         playposition++;
      }
   }
   
   if (!(counter & 0x1fff))
   {
      switch (drumbeat[drumcounter++])
      {
         case 'B':
            playposition = bassdrum;
            break;
         case 'S':
            playposition = snare;
            break;
         case 'H':
            playposition = hihat;
            break;
         default:
            break;
      };
      
      drumcounter &= 0x1F;
   }// counter & 0x3fff
   
   counter++;
}

int main(void)
{
   // HW init
   DDRD = (1 << PD4)|(1 << PD5);
   
   // Sampling Frequency is 8kHz, 1/(8kHz) = 125µsec
   // F_CPU = 16MegHz, 1/(16MegHz) = 62.5nsec
   // 125usec/62.5nsec = 2000 Mainclock ticks for one sample
   // let the OCR capable 16bit timer run from 0 to 255,
   // every 4th overflow catch a new sample
   // Getting proper Output out of OCR registers
   // in 4 timer cycle per sample...lets try
   ICR1 = 255;
   TCCR1A = (1<<COM1A1)|(0<<COM1A0)|(1<<COM1B1)|(0<<COM1B0)|(0<<FOC1A)|(0<<FOC1B)|(1<<WGM11)|(0<<WGM10);
   TCCR1B = (0<<ICNC1)|(0<<ICES1)|(0<<5)|(1<<WGM13)|(1<<WGM12)|(0<<CS12)|(0<<CS11)|(1<<CS10);
   OCR1A = 0x80;
   OCR1B = 0x80;
   TIMSK |= (1 << TOIE1);
   sei();
   
   while(1)
      ;   
   
   return(0);
}
